//+------------------------------------------------------------------+
//|   AdaptiveLaguerreFilter.mq4
//+------------------------------------------------------------------+

#property indicator_chart_window
#property indicator_buffers 3
#property indicator_color1  DarkGray
#property indicator_color2  Red
#property indicator_color3  ForestGreen
#property indicator_style1  STYLE_DASHDOT
#property indicator_width2  1
#property indicator_width3  1

//
//
//
//
//

extern string TimeFrame   = "current time frame";
extern int    LookBack    = 20;
extern int    Median      = 5;
extern int    Length      = 7;
extern int    PriceType   = PRICE_MEDIAN;
extern double Deviations  = 2;
extern bool   Interpolate = true;

//
//
//
//
//

double Filter[];
double UpperFilter[];
double LowerFilter[];
double Diff[];
double L0[];
double L1[];
double L2[];
double L3[];
double sortDiff[];
double alphas[];

//
//
//
//
//

int    timeFrame;
string IndicatorFileName;
bool   ReturningBars;
bool   CalculatingAlf;

//+------------------------------------------------------------------+
//|
//+------------------------------------------------------------------+
//
//
//
//

int init() 
{

   IndicatorBuffers(8);
   SetIndexBuffer(0,Filter);      SetIndexLabel(0,"ALF");
   SetIndexBuffer(1,UpperFilter); SetIndexLabel(1,"UpperALF");
   SetIndexBuffer(2,LowerFilter); SetIndexLabel(2,"LowerALF");
   SetIndexBuffer(3,Diff);
   SetIndexBuffer(4,L0);
   SetIndexBuffer(5,L1);
   SetIndexBuffer(6,L2);
   SetIndexBuffer(7,L3);
   ArrayResize(sortDiff,Median);

   //
   //
   //
   //
   //
         
      IndicatorFileName   = WindowExpertName();
      CalculatingAlf      = (TimeFrame=="CalculateAlf"); if (CalculatingAlf) return(0);
      ReturningBars       = (TimeFrame=="returnBars");   if (ReturningBars)  return(0);
      timeFrame           = stringToTimeFrame(TimeFrame);
   
   //
   //
   //
   //
   //
   
   IndicatorShortName("Adaptive Laguerre filter "+timeFrameToString(timeFrame)+" ("+LookBack+","+Median+","+PriceType+")");
   return (0);
}
  
//+------------------------------------------------------------------+
//|
//+------------------------------------------------------------------+
//
//
//
//
//
//

int start()
{
   int counted_bars = IndicatorCounted();
   int i,limit;
    
   if (counted_bars<0) return(-1);
   if (counted_bars>0) counted_bars--;
      limit = MathMin(Bars-counted_bars,Bars-1);
           if (ReturningBars)  { Filter[0] = limit; return(0); }

   //
   //
   //
   //
   //
   
   if (CalculatingAlf || timeFrame==Period())
   {
      for (i=limit;i>=0;i--)
      {
         double price = iMA(NULL,0,1,0,MODE_SMA,PriceType,i);
         Diff[i] = MathAbs(price - Filter[i+1]);

         //
         //
         //
         //
         //
      
         double HH = Diff[i];
         double LL = Diff[i];
            for (int j=0; j<LookBack; j++)
            {
               if (Diff[i+j] > HH) {HH = Diff[i+j];}
               if (Diff[i+j] < LL) {LL = Diff[i+j];}
            }

            double alpha = 0.5;
            if (HH!=LL)
            {
               for (j=0;j<Median;j++) sortDiff[j] = (Diff[i+j]-LL)/(HH-LL);
               ArraySort(sortDiff,WHOLE_ARRAY,0,MODE_ASCEND);
         
               if (MathMod(Median,2.0) != 0)
                     alpha = sortDiff[Median/2];         
               else  alpha = (sortDiff[Median/2]+sortDiff[(Median/2)-1])/2;
            }
      
         //
         //
         //
         //
         //
      
         L0[i]     = alpha*price + (1 - alpha)*L0[i+1];
         L1[i]     = -(1 - alpha)*L0[i] + L0[i+1] + (1 - alpha)*L1[i+1];
         L2[i]     = -(1 - alpha)*L1[i] + L1[i+1] + (1 - alpha)*L2[i+1];
         L3[i]     = -(1 - alpha)*L2[i] + L2[i+1] + (1 - alpha)*L3[i+1];
         Filter[i] = iNoLagMa((L0[i] + 2.0 * L1[i] + 2.0 * L2[i] + L3[i]) / 6.0,Length,alphas,0,i);
         
         //
         //
         //
         //
         //
         double sum=0.0;
         for(j=0; j<LookBack; j++)
         {
         double pr  = iMA(NULL,0,1,0,MODE_SMA,PRICE_CLOSE,i+j);
                sum+=(pr-Filter[i])*(pr-Filter[i]);
         }
         double StdDev = MathSqrt(sum/LookBack);

         UpperFilter[i]=Filter[i]+(StdDev*Deviations);
         LowerFilter[i]=Filter[i]-(StdDev*Deviations);
      }
      return(0);
   }  
   
   //
   //
   //
   //
   //
   
   if (timeFrame > Period()) limit = MathMax(limit,MathMin(Bars,iCustom(NULL,timeFrame,IndicatorFileName,"returnBars",0,0)*timeFrame/Period()));
   for (i=limit;i>=0;i--)
   {
      int y             = iBarShift(NULL,timeFrame,Time[i]);
         Filter[i]      = iCustom(NULL,timeFrame,IndicatorFileName,"CalculateAlf",LookBack,Median,Length,PriceType,Deviations,0,y);
         UpperFilter[i] = iCustom(NULL,timeFrame,IndicatorFileName,"CalculateAlf",LookBack,Median,Length,PriceType,Deviations,1,y);
         LowerFilter[i] = iCustom(NULL,timeFrame,IndicatorFileName,"CalculateAlf",LookBack,Median,Length,PriceType,Deviations,2,y);

         //
         //
         //
         //
         //
      
         if (timeFrame <= Period() || y==iBarShift(NULL,timeFrame,Time[i-1])) continue;
         if (!Interpolate) continue;

         //
         //
         //
         //
         //

         datetime time = iTime(NULL,timeFrame,y);
            for(int n = 1; i+n < Bars && Time[i+n] >= time; n++) continue;	
            double factor = 1.0 / n;
            for(int k = 1; k < n; k++) Filter[i+k] = k*factor*Filter[i+n] + (1.0-k*factor)*Filter[i];
   }
   return (0);   
}


//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
//
//
//
//
//


double values[][3];
double prices[][1];

#define Pi       3.14159265358979323846264338327950288
#define _length  0
#define _len     1
#define _weight  2

//
//
//
//
//

double iNoLagMa(double price, int length, double &alphasArray[], int forValue, int i)
{
   if (ArrayRange(prices,0) != Bars) ArrayResize(prices,Bars);
            int r = Bars-i-1;  prices[r][forValue]=price;
   if (length<3 || r<3) return(prices[r][forValue]);
   
   //
   //
   //
   //
   //
   
   if (ArrayRange(values,0)<(forValue+1) || values[forValue][_length] != length)
   {
      double Cycle = 4.0;
      double Coeff = 3.0*Pi;
      int    Phase = length-1;
      
      if (ArrayRange(values,0)<forValue+1) ArrayResize(values,forValue+1);

         values[forValue][_length] = length;
         values[forValue][_len]    = length*4 + Phase;  
         values[forValue][_weight] = 0;  
         ArrayResize(alphasArray,values[forValue][_len]);

         for (int k=0; k<values[forValue][_len]-1; k++)
         {
            if (k<=Phase-1) 
                 double t = 1.0 * k/(Phase-1);
            else        t = 1.0 + (k-Phase+1)*(2.0*Cycle-1.0)/(Cycle*length-1.0); 
            double beta = MathCos(Pi*t);
            double g = 1.0/(Coeff*t+1); if (t <= 0.5 ) g = 1;
      
            alphasArray[k]             = g * beta;
            values[forValue][_weight] += alphasArray[k];
         }
   }
   
   //
   //
   //
   //
   //
   
   if (values[forValue][_weight]>0)
   {
      int    len = values[forValue][_len];
      double sum = 0;
           for (k=0; k < len-1; k++) sum += alphasArray[k]*prices[r-k][forValue];      
           return( sum / values[forValue][_weight]);
   }
   else return(0);           
}

string sTfTable[] = {"M1","M5","M15","M30","H1","H4","D1","W1","MN"};
int    iTfTable[] = {1,5,15,30,60,240,1440,10080,43200};

//
//
//
//
//

int stringToTimeFrame(string tfs)
{
   tfs = StringUpperCase(tfs);
   for (int i=ArraySize(iTfTable)-1; i>=0; i--)
         if (tfs==sTfTable[i] || tfs==""+iTfTable[i]) return(MathMax(iTfTable[i],Period()));
                                                      return(Period());
}
string timeFrameToString(int tf)
{
   for (int i=ArraySize(iTfTable)-1; i>=0; i--) 
         if (tf==iTfTable[i]) return(sTfTable[i]);
                              return("");
}

//
//
//
//
//

string StringUpperCase(string str)
{
   string   s = str;

   for (int length=StringLen(str)-1; length>=0; length--)
   {
      int char = StringGetChar(s, length);
         if((char > 96 && char < 123) || (char > 223 && char < 256))
                     s = StringSetChar(s, length, char - 32);
         else if(char > -33 && char < 0)
                     s = StringSetChar(s, length, char + 224);
   }
   return(s);
}